
/*
 * pbrt source code Copyright(c) 1998-2007 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For educational use only; commercial use expressly forbidden.
 * NO WARRANTY, express or implied, for this software.
 * (See file License.txt for complete license)
 */

// Mirror_mcm.cpp*
#include "pbrt.h"
#include "material.h"
// Mirror_mcm Class Declarations
class Mirror_mcm : public Material {
public:
	// Mirror_mcm Public Methods
	Mirror_mcm(Reference<Texture<Spectrum> > r, Reference<Texture<float> > bump, Reference<Texture<int> > num, Reference<Texture<double> > temp) {
		Kr = r;
		bumpMap = bump;
		mcmNumeroDeLaSurface = num;
		mcmTemperature = temp;
	}
	BSDF *GetBSDF(const DifferentialGeometry &dgGeom, const DifferentialGeometry &dgShading) const;
private:
	// Mirror_mcm Private Data
	Reference<Texture<Spectrum> > Kr;
	Reference<Texture<float> > bumpMap;
	Reference<Texture<int> > mcmNumeroDeLaSurface;
	Reference<Texture<double> > mcmTemperature;
};
// Mirror_mcm Method Definitions
BSDF *Mirror_mcm::GetBSDF(const DifferentialGeometry &dgGeom, const DifferentialGeometry &dgShading) const {
	// Allocate _BSDF_, possibly doing bump-mapping with _bumpMap_
	DifferentialGeometry dgs;
	if (bumpMap)
		Bump(bumpMap, dgGeom, dgShading, &dgs);
	else
		dgs = dgShading;
	BSDF *bsdf = BSDF_ALLOC(BSDF)(dgs, dgGeom.nn);
	Spectrum R = Kr->Evaluate(dgs).Clamp();
	if (!R.Black())
		bsdf->Add(BSDF_ALLOC(SpecularReflection)(R,
			BSDF_ALLOC(FresnelNoOp)()));
	int num = mcmNumeroDeLaSurface->Evaluate(dgs);
	bsdf->set_mcmNumeroDeLaSurface(num);
	double temp = mcmTemperature->Evaluate(dgs);
	bsdf->set_mcmTemperature(temp);
	return bsdf;
}
extern "C" DLLEXPORT Material * CreateMaterial(const Transform &xform,
		const TextureParams &mp) {
	Reference<Texture<Spectrum> > Kr = mp.GetSpectrumTexture("Kr", Spectrum(1.f));
	Reference<Texture<float> > bumpMap = mp.GetFloatTexture("bumpmap", 0.f);
	Reference<Texture<int> > mcmNumeroDeLaSurface = mp.GetIntTexture("numero", -1);
	Reference<Texture<double> > mcmTemperature = mp.GetDoubleTexture("temperature", -1.);
	return new Mirror_mcm(Kr, bumpMap, mcmNumeroDeLaSurface, mcmTemperature);
}